<?php
/* ==================================================
 *   Ktai Admin Edit Page
 *   based on wp-admin/edit-pages.php of WP 2.7
   ================================================== */

define ('KS_HEADER_FOOTER_SIZE', 3000);
define ('KS_COLUMN_SIZE', 256);
require dirname(__FILE__) . '/admin.php';
$title = __('Edit Pages', 'ktai_style');
$parent_file = 'edit-pages.php';
$View = new KS_Admin_Edit_Pages($Ktai_Style, $KS_Admin);
$View->output();
exit;

/* ==================================================
 *   KS_Admin_Edit_Pages class
   ================================================== */

class KS_Admin_Edit_Pages {
	private $base;
	private $admin;
	private $search;

// ==================================================
public function __construct($base, $admin) {
	$this->base  = $base;
	$this->admin = $admin;
	$this->search = isset($_GET['s']) ? stripslashes($_GET['s']) : '';
}

// ==================================================
private function edit_pages_query() {
	$post_stati  = array(	//	array( adj, noun )
		'publish' => array(__('Published'), __('Published pages'), __ngettext_noop('Published (%s)', 'Published (%s)')),
		'future' => array(__('Scheduled'), __('Scheduled pages'), __ngettext_noop('Scheduled (%s)', 'Scheduled (%s)')),
		'pending' => array(__('Pending Review'), __('Pending pages'), __ngettext_noop('Pending Review (%s)', 'Pending Review (%s)')),
		'draft' => array(__('Draft'), _c('Drafts|manage posts header'), __ngettext_noop('Draft (%s)', 'Drafts (%s)')),
		'private' => array(__('Private'), __('Private pages'), __ngettext_noop('Private (%s)', 'Private (%s)'))
	);
	$post_stati = apply_filters('post_stati', $post_stati);

	$per_page = intval(($this->base->get('page_size') - KS_HEADER_FOOTER_SIZE) / KS_COLUMN_SIZE);
	if ($per_page < 1) {
		$per_page = 1;
	} elseif ($per_page > 15) {
		$per_page = 15;
	}
	$pagenum = isset($_GET['paged']) ? intval($_GET['paged']) : 1;

	$query = array(
		'post_type' => 'page', 
		'what_to_show' => 'posts',
		'post_status' => 'any',
		'posts_per_page' => -1, 
		'posts_per_archive_page' => -1, 
		'orderby' => 'menu_order title', 
		'order' => 'ASC',
		's' => $this->search,
	);

	if ( isset($_GET['post_status'])) {
		$post_status = stripslashes($_GET['post_status']);
		if ( in_array($post_status, array_keys($post_stati)) ) {
			$query['post_status'] = $post_status;
			$query['perm'] = 'readable';
		}
	}
	$query = apply_filters('manage_pages_query', $query);
	query_posts($query);

	return array($post_stati, $pagenum, $per_page);
}

// ==================================================
public function status_menu($post_stati) {
	global $parent_file, $submenu, $submenu_file;

	$num_posts = wp_count_posts('page', 'readable');
	$num_total = array_sum( (array) $num_posts);

	$link = $submenu_file = 'edit-pages.php';
	$submenu[$parent_file] = array(
		array(($this->base->check_wp_version(2.7) ? __('Add New Page') : __('Write a New Page')), 'edit_pages', 'page-new.php'),
		array(sprintf(__('All (%s)', 'ktai_style'), $num_total), 'edit_pages', $link),
	);
	$post_status_label = __('Pages');
	$avail_post_stati = get_available_post_statuses('page');
	foreach ( $post_stati as $status => $label ) {
		if ( !in_array( $status, $avail_post_stati ) ) {
			continue;
		}
		if ( empty( $num_posts->$status ) ) {
			continue;
		}
		$link = add_query_arg( 'post_status', $status, $link );
		$submenu[$parent_file][] = array(sprintf(__ngettext( $label[2][0], $label[2][1], $num_posts->$status, 'ktai_style'), number_format_i18n($num_posts->$status)), 'edit_pages', $link);
		if ( str_replace( 'all', '', $status ) == $_GET['post_status'] ) {
			$submenu_file = $link;
			$post_status_label = $label[1];
			$num_total = $num_posts->$status;
		}
	}
	return array($post_status_label, $num_total);
}

// ==================================================
public function output() {
	global $title, $parent_file;
	list($post_stati, $pagenum, $per_page) = $this->edit_pages_query();

	include dirname(__FILE__) . '/admin-header.php';
	list($post_status_label, $total_posts) = $this->status_menu($post_stati);
	if ($this->search) {
		echo '<h2>' . sprintf( __('Search results for &#8220;%s&#8221;'), wp_specialchars($this->search) ) . '</h2>';
	}
	if ( isset($_GET['posted']) && $_GET['posted'] ) {
		$_GET['posted'] = (int) $_GET['posted'];
		?><p><font color="teal"><?php _e('Your page has been saved.'); ?></font></p><?php
		$_SERVER['REQUEST_URI'] = remove_query_arg(array('posted'), $_SERVER['REQUEST_URI']);
	}
?>
<form action="" method="get"><div>
<?php $this->admin->sid_field(); ?>
<?php _e('Search'); ?><input type="text" name="ks" value="<?php echo wp_specialchars($this->search); ?>" size="17" /><br />
<?php if (isset($_GET['post_status'])) { ?>
<input type="hidden" name="post_status" value="<?php echo attribute_escape($_GET['post_status']) ?>" />
<?php } ?>
<input type="submit" value="<?php _e('Filter', 'ktai_style'); ?>" /></div></form>
<?php 
	if (have_posts()) {
		$this->page_rows($pagenum, $per_page);
		$num_pages = ceil($GLOBALS['wp_query']->post_count / $per_page);
		$page_links = paginate_links( array(
			'base' => add_query_arg( 'paged', '%#%' ),
			'format' => '',
			'total' => $num_pages,
			'current' => $pagenum,
			'prev_text' => __('<img localsrc="7" alt="&laquo;" />Previous Page', 'ktai_style'),
			'next_text' => __('Next Page<img localsrc="8" alt="&raquo;" />', 'ktai_style'),
		));
		if ($page_links) {
			$page_links = $this->base->filter_tags($page_links);
			$page_links = preg_replace('/ class=([\'"])[-_ \w]+\1/', '', $page_links); //"
			$page_links = str_replace("\n", ' ', $page_links);
			$page_links = str_replace(ks_admin_url(false), '', $page_links);
			echo '<br /><div>' . $page_links . '</div>';
		}
	} else { ?>
<p><?php _e('No pages found.') ?></p>
<?php
	} // have_posts()
	include dirname(__FILE__) . '/admin-footer.php';
}

// ==================================================
private function page_rows($pagenum = 1, $per_page = 15) {
	global $wpdb, $wp_query;
	$level = 0;
	$pages =& $wp_query->posts;
	if ( ! $pages ) {
		return false;
	}
	/*
	 * arrange pages into two parts: top level pages and children_pages
	 * children_pages is two dimensional array, eg.
	 * children_pages[10][] contains all sub-pages whose parent is 10.
	 * It only takes O(N) to arrange this and it takes O(1) for subsequent lookup operations
	 * If searching, ignore hierarchy and treat everything as top level
	 */
	if ( empty($this->search) ) {

		$top_level_pages = array();
		$children_pages = array();

		foreach ( $pages as $page ) {

			// catch and repair bad pages
			if ( $page->post_parent == $page->ID ) {
				$page->post_parent = 0;
				$wpdb->query( $wpdb->prepare("UPDATE $wpdb->posts SET post_parent = '0' WHERE ID = %d", $page->ID) );
				clean_page_cache( $page->ID );
			}

			if ( 0 == $page->post_parent )
				$top_level_pages[] = $page;
			else
				$children_pages[ $page->post_parent ][] = $page;
		}

		$pages = &$top_level_pages;
	}

	$count = 0;
	$start = ($pagenum - 1) * $per_page;
	$end = $start + $per_page;
?><dl><?php 
	foreach ( $pages as $page ) {
		if ( $count >= $end )
			break;

		if ( $count >= $start )
			echo $this->display_page_row( $page, $level );

		$count++;

		if ( isset($children_pages) )
			$this->_page_rows( $children_pages, $count, $page->ID, $level + 1, $pagenum, $per_page );
	}

	// if it is the last pagenum and there are orphaned pages, display them with paging as well
	if ( isset($children_pages) && $count < $end ){
		foreach( $children_pages as $orphans ){
			foreach ( $orphans as $op ) {
				if ( $count >= $end )
					break;
				if ( $count >= $start )
					echo $this->display_page_row( $op, 0 );
				$count++;
			}
		}
	}
?></dl><?php 
}

// ==================================================
private function _page_rows( &$children_pages, &$count, $parent, $level, $pagenum, $per_page ) {

	if ( ! isset( $children_pages[$parent] ) )
		return;

	$start = ($pagenum - 1) * $per_page;
	$end = $start + $per_page;

	foreach ( $children_pages[$parent] as $page ) {

		if ( $count >= $end )
			break;

		// If the page starts in a subtree, print the parents.
		if ( $count == $start && $page->post_parent > 0 ) {
			$my_parents = array();
			$my_parent = $page->post_parent;
			while ( $my_parent) {
				$my_parent = get_post($my_parent);
				$my_parents[] = $my_parent;
				if ( !$my_parent->post_parent )
					break;
				$my_parent = $my_parent->post_parent;
			}
			$num_parents = count($my_parents);
			while( $my_parent = array_pop($my_parents) ) {
				echo $this->display_page_row( $my_parent, $level - $num_parents );
				$num_parents--;
			}
		}

		if ( $count >= $start )
			echo $this->display_page_row( $page, $level );

		$count++;

		$this->_page_rows( $children_pages, $count, $page->ID, $level + 1, $pagenum, $per_page );
	}

	unset( $children_pages[$parent] ); //required in order to keep track of orphans
}

// ==================================================
function display_page_row( $page, $level = 0 ) {
	global $post;
	$pages_columns = array('dt', 'title', '/dt', 'dd', 'author', 'date', 'comments', '/dd');

	$post = $page;
	setup_postdata($page);
	foreach ($pages_columns as $column_name) {
		switch ($column_name) {
		case 'dt':
			echo '<dt>';
			break;
		case '/dt':
			echo '</dt>';
			break;
		case 'dd':
			echo '<dd>';
			break;
		case '/dd':
			echo '</dd>';
			break;
		case 'dd-small':
			echo '<dd><small>';
			break;
		case '/dd-small':
			echo '</small></dd>';
			break;
		case 'title':
			$title = $this->admin->draft_or_post_title();
			$pad = str_repeat('&#8212; ', $level);
			if (current_user_can('edit_post', $page->ID)) {
				$title = sprintf(__('<img localsrc="149" alt="" /><a href="%1$s">%2$s</a>', 'ktai_style'), $this->admin->add_sid(get_edit_post_link($page->ID, 'url')), $title);
			}
			printf(_c('%1$5d:%2$s%3$s%4$s|page_row_title', 'ktai_style'), $post->ID, $pad, $title, $this->admin->get_post_states($page));
			break;
		case 'date':
			if ('draft' === $_GET['post_status'] || 'pending' === $_GET['post_status']) {
				if ('0000-00-00 00:00:00' == $post->post_modified) {
					$time = __('Never');
				} else {
					$time = ks_get_mod_time();
				}
			} else {
				if ('0000-00-00 00:00:00' == $post->post_date) {
					$time = __('Unpublished');
				} else {
					$time = ks_get_time();
				}
			}
			printf(__('<img localsrc="46" alt=" at " />%s', 'ktai_style'), '<font color="' . ks_option('ks_date_color') . '">' . $time . '</font>');
			break;
		case 'author':
			printf(__('<img localsrc="68" alt="by " />%s', 'ktai_style'), get_the_author());
			break;
		case 'comments':
			$link = '<a href="' . $this->admin->add_sid("edit-comments.php?p={$post->ID}") . '">';
			echo ' <img localsrc="86" alt="[' . __('Comments') . '] " />';
			comments_number(
				($comment_pending_count[$post->ID] ? ($link . __('0') . '</a>') : __('0')), 
				$link . __('1') . '</a>', 
				$link . __('%') . '</a>'
			);
			if ( $comment_pending_count[$post->ID] ) {
				echo '<img localsrc="2" alt="[' . sprintf(__('%s pending'), number_format( $comment_pending_count[$post->ID]) ) . ']" />';
			}
			break;
		}
	}
}

// ===== End of class ====================
}
?>